/*
    DSync.cpp

         Disk Sync utility for Windows
   
    CONTACTS
         E-mail regarding any portion of the Disk Sync should be
         directed to the developer:
                 alter@alter.org.ua
    HISTORY
         May, 2003 - Alter
             Sync for all types of media
         Oct, 2004 - Alter
             Sync for single volume
 */

#include <stdio.h>
#include <windows.h>
#include <fcntl.h>
#include "getopt.h"
#include <stdlib.h>
#include <string.h>
#include <time.h>

/* These should be changed if you make any code changes */
#define VERSION_MAJOR   0
#define VERSION_MINOR   0
#define VERSION_BUILD   0x1

/*
 * Command line option token values.
 *  0x0000-0x00ff   Single characters
 *  0x1000-0x1fff   Long switches (no arg)
 *  0x2000-0x2fff   Long settings (arg required)
 */
#define OPT_H           0x0001

#define OPT_HELP        0x1000
#define OPT_VERBOSE     0x1001
#define OPT_VERSION     0x1002

struct option long_options[] = {
    { "?", no_argument, NULL, OPT_H },
    { "h", no_argument, NULL, OPT_H },
    { "help", no_argument, NULL, OPT_HELP },
    { "verb", no_argument, NULL, OPT_VERBOSE },
    { "v", no_argument, NULL, OPT_VERSION },
    { NULL, no_argument, NULL, NULL }
};

/* Command line globals */
char *opt_filename = NULL;

void
usage(void)
{
    fprintf(stderr, "Disk Sync (WinNT tools) "
        "%d.%d (%d)\n", 0 /* VERSION_MAJOR*/,
        1 /*VERSION_MINOR*/, 2/*VERSION_BUILD*/);
    fprintf(stderr,
        "Usage:\n"
        "\tDSync [options] FILE/DEVICE\n"
        "Options:\n"
        "\t--help, --h, --?          display this screen\n"
        "\t--verb                    verbose mode\n"
        "\t--v                       version\n"
        "\n"

        "Example:\n"
        "\tDSync X:\n"
    );
}

void
parse_args(int argc, char *argv[])
{
    int retval;
    while (argc > 1) {
        retval = getopt_long(argc, argv, "V", long_options, NULL);
        if(retval == EOF)
            break;
        switch (retval)
        {
            case OPT_HELP:
            case OPT_H:
                usage();
                exit(0);
                break;
            case OPT_VERBOSE:
                break;
            default:
                fprintf(stderr, "Try `DSync --help' "
                    "for more information\n");
                exit(-2);
        }
    }

    if (optind && optind == argc-1)
        opt_filename = argv[optind];
    else {
        opt_filename = NULL;
/*        fprintf(stderr, "Try `DSync --help' for more information\n");
        exit(-3);*/
    }
}


int
sync_device(char* opt_filename)
{
    CCHAR deviceNameBuffer[0x200];
    ULONG status;
    HANDLE h;

    if(strlen(opt_filename) > sizeof(deviceNameBuffer) - 0x08) {
        printf("Too long volume name:\n\t%s\n",
            opt_filename);
        return (-2);
    }
    sprintf(deviceNameBuffer, "\\\\.\\%s", opt_filename);
    h = CreateFile(deviceNameBuffer, 0 | GENERIC_READ | GENERIC_WRITE,
                   0 | FILE_SHARE_READ | FILE_SHARE_WRITE,
                   NULL,
                   OPEN_EXISTING,
                   FILE_ATTRIBUTE_NORMAL/* | FILE_FLAG_NO_BUFFERING*/,  NULL);
    if(h == ((HANDLE)-1)) {
        status = GetLastError();
        printf("Can't open volume %s.\n\tError code %8.8x\n",
            opt_filename, status);
        return (-1);
    }

    if(!FlushFileBuffers(h)) {
        status = GetLastError();
        printf("Can't sync volume %s.\n\tError code %8.8x\n",
            opt_filename, status);
        return (-4);
    }
    printf("Volume %s is sync'ed successfully\n", opt_filename);
    return 0;
}

void
main(int argc, char *argv[])
{
    char DrvLetter[] = {'X',':',0};
    ULONG status;
    ULONG DrvLetterMap = 0;
    ULONG i;

    printf("Volume Sync utility v 1.2. Copiright (c) Alexander A. Telyatnikov\n");

    parse_args(argc, argv);

    if (opt_filename) {
        status = sync_device(opt_filename);
        exit(status);
    }

    DrvLetterMap = GetLogicalDrives();
    printf("Syncing all volumes...\n");
    for(i=2; i<32; i++) {
        if(!(DrvLetterMap & (1<<i)))
            continue;
        //printf("Syncing all volumes...\n");
        DrvLetter[0] = 'A' + (char)i;
        status = sync_device(&DrvLetter[0]);
    }
    exit(0);
}

